@@ -19,3 +19,5 @@ class OfflineQueryGivensVersion(IntEnum):
 
 TS_COL_NAME = "__ts__"
 INDEX_COL_NAME = "__index__"
+PKEY_COL_NAME = "__id__"
+OBSERVED_AT_COL_NAME = "__oat__"
