@@ -862,6 +862,7 @@ class DatasetRevisionImpl(DatasetRevision):
         ignore_errors: bool = False,
         show_progress: bool | ellipsis = ...,
         timeout: float | timedelta | ellipsis | None = ...,
+        skip_failed_shards: bool = False,
         caller_name: str = "get_data_as_polars",
     ) -> pl.LazyFrame:
         context = OfflineQueryContext(environment=self.environment)
@@ -869,14 +870,15 @@ class DatasetRevisionImpl(DatasetRevision):
             self._hydrate(caller_method=caller_name, timeout=timeout, show_progress=show_progress)
         return self._client.load_dataset(
             job_id=self.revision_id,
-            context=context,
-            output_id=output_id,
             outputs=self.outputs,
+            output_id=output_id,
             output_ts=output_ts,
+            context=context,
             branch=self.branch,
             ignore_errors=ignore_errors,
             query_inputs=False,
             return_type="polars_lazyframe",
+            skip_failed_shards=skip_failed_shards,
         )
 
     def arrow_schema(
@@ -983,6 +985,7 @@ class DatasetRevisionImpl(DatasetRevision):
         ignore_errors: bool = False,
         show_progress: bool | ellipsis = ...,
         timeout: float | timedelta | ellipsis | None = ...,
+        skip_failed_shards: bool = False,
         caller_name: str = "get_data_as_pandas",
     ) -> pd.DataFrame:
         context = OfflineQueryContext(environment=self.environment)
@@ -999,6 +1002,7 @@ class DatasetRevisionImpl(DatasetRevision):
             ignore_errors=ignore_errors,
             query_inputs=False,
             return_type="pandas",
+            skip_failed_shards=skip_failed_shards,
         )
 
     def to_pandas(
@@ -1006,8 +1010,9 @@ class DatasetRevisionImpl(DatasetRevision):
         output_id: bool = False,
         output_ts: bool | str = False,
         ignore_errors: bool = False,
-        timeout: float | timedelta | ellipsis | None = ...,
         show_progress: bool | ellipsis = ...,
+        timeout: float | timedelta | ellipsis | None = ...,
+        skip_failed_shards: bool = False,
         caller_name: str = "to_pandas",
     ) -> pd.DataFrame:
         return self.get_data_as_pandas(
@@ -1017,6 +1022,7 @@ class DatasetRevisionImpl(DatasetRevision):
             timeout=timeout,
             show_progress=show_progress,
             caller_name=caller_name,
+            skip_failed_shards=skip_failed_shards,
         )
 
     def get_data_as_dataframe(
@@ -1595,6 +1601,7 @@ class DatasetImpl(Dataset):
         ignore_errors: bool = False,
         show_progress: bool | ellipsis = ...,
         timeout: float | timedelta | ellipsis | None = ...,
+        skip_failed_shards: bool = False,
         caller_name: str = "get_data_as_pandas",
     ) -> pd.DataFrame:
         if len(self.revisions) == 0:
@@ -1605,6 +1612,7 @@ class DatasetImpl(Dataset):
             ignore_errors=ignore_errors,
             timeout=timeout,
             show_progress=show_progress,
+            skip_failed_shards=skip_failed_shards,
             caller_name=caller_name,
         )
 
@@ -1615,6 +1623,7 @@ class DatasetImpl(Dataset):
         ignore_errors: bool = False,
         show_progress: bool | ellipsis = ...,
         timeout: float | timedelta | ellipsis | None = ...,
+        skip_failed_shards: bool = False,
         caller_name: str = "to_pandas",
     ) -> pd.DataFrame:
         return self.get_data_as_pandas(
@@ -1623,6 +1632,7 @@ class DatasetImpl(Dataset):
             ignore_errors=ignore_errors,
             timeout=timeout,
             show_progress=show_progress,
+            skip_failed_shards=skip_failed_shards,
             caller_name=caller_name,
         )
 
