@@ -470,10 +470,6 @@ class OnlineQueryRequest(BaseModel):
     context: Optional[OnlineQueryContext] = None
     include_meta: bool = True
     explain: Union[bool, Literal["only"]] = False
-    skip_online_storage: Optional[bool] = False
-    skip_offline_storage: Optional[bool] = False
-    skip_metrics_storage: Optional[bool] = False
-    skip_cache_lookups: Optional[bool] = False
     correlation_id: Optional[str] = None
     query_name: Optional[str] = None
     query_name_version: Optional[str] = None
@@ -508,10 +504,6 @@ class OnlineQueryManyRequest(BaseModel):
     context: Optional[OnlineQueryContext] = None
     include_meta: bool = True
     explain: bool = False
-    skip_online_storage: Optional[bool] = False
-    skip_offline_storage: Optional[bool] = False
-    skip_metrics_storage: Optional[bool] = False
-    skip_cache_lookups: Optional[bool] = False
     correlation_id: Optional[str] = None
     query_name: Optional[str] = None
     query_name_version: Optional[str] = None
@@ -1735,8 +1727,8 @@ class PingResponse(BaseModel):
 
 
 class ModelUploadUrlResponse(BaseModel):
+    model_artifact_id: str
     upload_urls: Mapping[str, str]
-    success: bool
 
 
 class RegisterModelResponse(BaseModel):
