@@ -83,7 +83,10 @@ def validate_df_schema(underlying: Union[pl.DataFrame, pl.LazyFrame]):
             underlying = underlying.with_columns(
                 _generate_empty_series_for_dtype(root_fqn, expected_dtype, len(underlying))
             )
-        elif isinstance(expected_dtype, pl.List) and actual_dtype == pl.Utf8:
+        elif (
+            isinstance(expected_dtype, pl.List)
+            and actual_dtype == pl.Utf8  # pyright: ignore[reportUnnecessaryComparison]
+        ):
             col = pl.col(root_fqn).str.json_extract(expected_dtype)
             try:
                 underlying = underlying.with_columns(col.cast(expected_dtype))
@@ -116,7 +119,7 @@ def validate_df_schema(underlying: Union[pl.DataFrame, pl.LazyFrame]):
                     ) from e
         else:
             try:
-                if actual_dtype == pl.Utf8:
+                if actual_dtype == pl.Utf8:  # pyright: ignore[reportUnnecessaryComparison]
                     if isinstance(expected_dtype, pl.Datetime):
                         # tzinfo = None if expected_dtype.time_zone is None else zoneinfo.ZoneInfo(expected_dtype.time_zone)
                         underlying = underlying.with_columns(pl.col(root_fqn).str.strptime(pl.Datetime).alias(root_fqn))
