@@ -88,10 +88,10 @@ def _validate_max_staleness(
     if feature.maxStaleness is not None and feature.maxStaleness != "infinity":
         try:
             parse_chalk_duration(feature.maxStaleness)
-        except:
+        except ValueError as e:
             builder.add_error(
                 header=f'Could not parse max_staleness for feature "{feature.id.fqn}"',
-                subheader=f'Received "{feature.maxStaleness}". See https://docs.chalk.ai/docs/duration for valid formats.',
+                subheader=f"Invalid max staleness. {e.args[0]}",
             )
 
 
@@ -115,15 +115,19 @@ def _validate_no_feature_times_as_input(
 ):
     for i, inp in enumerate(resolver.inputs or []):
         if inp.underlying.fqn not in fqn_to_feature:
+            message = (
+                f"Resolver '{resolver.fqn}' requires an unrecognized feature '{inp.underlying.fqn}'. "
+                + f"This may have happened since the import of feature class '{inp.underlying.namespace}' may have failed"
+            )
             lsp_builder.add_diagnostic(
-                message=f"Resolver '{resolver.fqn}' requires an unrecognized feature '{inp.underlying.fqn}'",
+                message=message,
                 code="135",
                 range=lsp_builder.function_arg_annotation_by_index(i),
                 label="invalid resolver input",
             )
             builder.add_error(
-                header=f"Resolver '{resolver.fqn}' requires an unknown feature '{inp.underlying.fqn}'",
-                subheader="Please check the inputs to this resolver, and make sure they are all features.",
+                header=message,
+                subheader="Please check the feature inputs to this resolver were imported correctly.",
             )
         elif fqn_to_feature[inp.underlying.fqn].featureTimeKind is not None and resolver.kind != "offline":
             lsp_builder.add_diagnostic(
@@ -244,11 +248,11 @@ def _validate_resolver_output(
             code="132",
             range=lsp_builder.function_return_annotation(),
             label="invalid resolver output",
-            code_href="https://docs.chalk.ai/docs/resolver-outputs",
+            code_href="https://docs.chalk.ai/docs/python-resolvers#outputs",
         )
         builder.add_error(
             header=f'Resolver "{resolver.fqn}" does not define outputs.',
-            subheader="See https://docs.chalk.ai/docs/resolver-outputs for information about valid resolver outputs.",
+            subheader="See https://docs.chalk.ai/docs/python-resolvers#outputs for information about valid resolver outputs.",
         )
 
     if len(dataframes) > 1:
@@ -257,11 +261,11 @@ def _validate_resolver_output(
             code="133",
             range=lsp_builder.function_return_annotation(),
             label="invalid resolver output",
-            code_href="https://docs.chalk.ai/docs/resolver-outputs",
+            code_href="https://docs.chalk.ai/docs/python-resolvers#outputs",
         )
         builder.add_error(
             header=f'Resolver "{resolver.fqn}" defines multiple DataFrames as output.',
-            subheader="See https://docs.chalk.ai/docs/resolver-outputs for information about valid resolver outputs.",
+            subheader="See https://docs.chalk.ai/docs/python-resolvers#outputs for information about valid resolver outputs.",
         )
 
     if len(features) > 0 and len(dataframes) > 0:
@@ -273,20 +277,26 @@ def _validate_resolver_output(
             code="134",
             range=lsp_builder.function_return_annotation(),
             label="invalid resolver output",
-            code_href="https://docs.chalk.ai/docs/resolver-outputs",
+            code_href="https://docs.chalk.ai/docs/python-resolvers#outputs",
         )
         builder.add_error(
             header=f'Resolver "{resolver.fqn}" returns both relationships and scalar features.',
-            subheader="See https://docs.chalk.ai/docs/resolver-outputs for information about valid resolver outputs.",
+            subheader="See https://docs.chalk.ai/docs/python-resolvers#outputs for information about valid resolver outputs.",
         )
 
     # Want to validate this but only have feature id instead of feature
     # if len(resolver.inputs) == 0 and len(dataframes) == 1 and len([pkey for pkey in dataframes[0].columns if pkey.primary]) != 1:
     #     builder.add_error(
     #         header=f'Resolver "{resolver.fqn}" must return a primary feature',
-    #         subheader="See https://docs.chalk.ai/docs/resolver-outputs for information about valid resolver outputs.",
+    #         subheader="See https://docs.chalk.ai/docs/python-resolvers#outputs for information about valid resolver outputs.",
     #     )
-
+    # for output_feature in features:
+    #     if is_underlying_has_many(output_feature):
+    #         if len([pkey for pkey in output_feature.underlying.df.columns if pkey.primary]) != 1:
+    #         builder.add_error(
+    #             header=f'Resolver "{resolver.fqn}" must return a primary feature in has-many outputs',
+    #             subheader="See https://docs.chalk.ai/docs/python-resolvers#outputs for information about valid resolver outputs.",
+    #         )
     # Bring this back.
 
 
@@ -297,7 +307,7 @@ def _validate_resolver_output(
 #                 header=f'Resolver "{resolver.fqn}" outputs features from multiple namespaces.',
 #                 subheader=f"""All outputs of a resolver should be rooted in the same namespace.
 # The resolver "{resolver.fqn}" outputs features in the namespaces {oxford_comma_list(namespaces)}.
-# See https://docs.chalk.ai/docs/resolver-outputs for information about valid resolver outputs.
+# See https://docs.chalk.ai/docs/python-resolvers#outputs for information about valid resolver outputs.
 #
 # {resolver.functionDefinition}
 # """,
